package cz.cas.icpf.hroch486.selfml;

import java.io.IOException;
import java.io.File;
import java.io.FileOutputStream;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import org.dom4j.Node;
import org.dom4j.Element;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.DocumentException;
import org.dom4j.XPath;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.dom4j.io.OutputFormat;

/**
Main EDIP class, all methods are static.
@author Zdenek Wagner
 */
public final class Edip {

    /** Namespace of the internal XML document */
    public static final String INTERNAL_NS = "http://hroch486.icpf.cas.cz/ns/selfml/internal";

    /** Name of the property containing the config file name */
    private static final String PARSERS_CONFIG = "edip.config";
    /** Default config file name locadet in java.home */
    private static final String CFG_TAG = "SELFMLconfig";
    /** Config element containing the list of spreadsheet parsers */
    private static final String CFG_PARSER_TAG = "parsers";
    /** Config element containing the list of property builders */
    private static final String CFG_BUILDER_TAG = "builders";

    /** Configuration document, readconfig must be called if <b>null</b> */
    private static Document cfgDocument = null;
    /** Configuration root element */
    private static Element cfgElement;
    /** Name of the configuration file */
    private static String cfgName;


/**
Add spreadsheet parser(s) and property builders to the configuration and write the configuration to
the file
@param names the array of fully qualified class names, the first item is ignored
*/
    public static void add(String names[]) {
        readConfig();
        for (int i = 1;  i < names.length;  i++) {
            Object object = newInstance(names[i]);
            if (object != null) {
                if (object instanceof SpreadsheetParser) {
                    SpreadsheetParser parser = (SpreadsheetParser)object;
                    addParser(names[i], parser.getNamespaceURI(), parser.getElementName());
                } else if (object instanceof PropertyBuilder) {
                    PropertyBuilder builder = (PropertyBuilder)object;
                    addBuilder(names[i]);
                }
            }
        }
        writeConfig();
        viewConfig();
    }


/**
Adds all property builders specified by the class
@param classname fully qualified class name
*/
    public static void addBuilder(String classname) {
        PropertyBuilder builder = newBuilder(classname);
        if (builder != null) addBuilder(builder, classname);
    }


/**
Adds all property builders for the supplied builder
@param builder property builder
*/
    public static void addBuilder(PropertyBuilder builder) {
        addBuilder(builder, builder.getClass().getName());
    }

/**
Adds all property builders for the supplied builder and class name
@param builder property builder
@param classname fully qualified class name
*/
    private static void addBuilder(PropertyBuilder builder, String classname) {
        for (Enumeration proplist = builder.getPropertyNames();
                         proplist.hasMoreElements(); ) {
            addBuilder(classname, (String)proplist.nextElement());
        }
    }

/**
Adds a property builder for the specified property
@param property property type
@param classname fully qualified class name
*/
    public static void addBuilder(String classname, String property) {
        removeBuilderByProperty(property);
        Element builders = getBuildersNode();
        Element ent = builders.addElement(property).addAttribute("classname", classname);
    }

/**
Adds a specified parser for specified NS URI and element name
@param classname fully qualified class name
@param ns spreadsheet namespace URI
@param elem spreadsheet root element name
*/
    public static void addParser(String classname, String ns, String elem) {
        removeParser(ns, elem);
        Element parsers = getParsersNode();
        Element ent = parsers.addElement(elem)
                             .addAttribute("ns", ns)
                             .addAttribute("classname", classname);
    }


/** Cleanup missing classes */
    public static void cleanup() {
        readConfig();
/****************************************************************************
        NodeList children = cfgElement.getChildNodes();
        for (int i = 0;  i < children.getLength();  i++) {
            if (children.item(i).getNodeType() != Node.ELEMENT_NODE) continue;
            Element elem = (Element)children.item(i);
            String name = elem.getTagName();
            String classname = elem.getAttribute("classname");
            try {
                if (classname == null)
                    throw new EdipException("Missing classname attribute");
                if (name.equals(CFG_PARSER_TAG)) { // parsers
                    SpreadsheetParser parser = newParser(classname);
                    if (parser == null)
                        throw new EdipException("Parser for " + classname
                                                      + " not found.");
                } else if (name.equals(CFG_BUILDER_TAG)) { // builders
                    PropertyBuilder builder = newBuilder(classname);
                    if (builder == null)
                        throw new EdipException("Builder for " + classname
                                                      + " not found.");
                } else
                    throw new EdipException("Unknown element");
            } catch(EdipException e) {
                System.out.println(e.getMessage());
                try {
                    cfgElement.removeChild(elem);
                } catch (DOMException de) {
                    System.err.println(de.getMessage());
                }
            }
        }
**************************************************************************/
        viewConfig();
        writeConfig();
    }


/**
Finds the class name for the property builder
@param property property type
@return class name or <b>null</b> if nothing found
*/
    public static String findBuilder(String property) {
        return findXPathCfg("/SELFMLconfig/builders/" + property);
    }


/**
Finds al property builders for the specified property
@param property property type
@return list of property builder nodes
*/
    public static List findBuilders(String property) {
        return findXconfig("/SELFMLconfig/builders/" + property);
    }


/**
Finds the class name for the spreadsheet parser
@param ns spreadsheet namespace URI (or an empty string, never use <b>null</b>)
@param name spreadsheet root element name
@return class name or <b>null</b> if nothing found
*/
    public static String findParser(String ns, String name) {
        return findXPathCfg("/SELFMLconfig/parsers/" + name + "[@ns='" + ns + "']");
    }


/**
Finds all spreadsheet parsers for specified namespace URI and element name
@param ns spreadsheet namespace URI (or an empty string, never use <b>null</b>)
@param name spreadsheet root element name
@return list of spreadsheet parser nodes
*/
    public static List findParsers(String ns, String name) {
        return findXconfig("/SELFMLconfig/parsers/" + name + "[@ns='" + ns + "']");
    }


/**
Finds the configuration element defined by the XPath, returns only the first element
@param expression XPath expression
@return value of the classname attribute of the element or <b>null</b> if nothing found
*/

    private static String findXPathCfg(String expression) {
        List results = findXconfig(expression);
        return results.size() == 0 ? null :
            ((Element)results.get(0)).attributeValue("classname");
    }

/**
Returns all configuration nodes for a given XPath expression
@param expression XPath expression
@return list of nodes
*/

    private static List findXconfig(String expression) {
        readConfig();
        XPath xpathSelector = DocumentHelper.createXPath(expression);
        return xpathSelector.selectNodes(cfgDocument);
    }

/**
Returns the config node of the specified name or creates a new one
@param name name of the node which represents either parsers or builders
@return corresponding element or <b>null</b> if it cannot be created but this should not happen
with dom4j
*/
    private static Element getConfigNode(String name) {
        readConfig();
        Element ent = null;
        XPath xpathSelector = DocumentHelper.createXPath("/SELFMLconfig/" + name);
        try {
            List results = xpathSelector.selectNodes(cfgDocument);
            if (results.size() > 1) throw new EdipException("Corrupted configuration");
            if (results.size() == 1) ent = (Element)results.get(0);
            else ent = cfgElement.addElement(name);
        } catch (EdipException e) {
            e.printStackTrace();
        }
        return ent;
    }


    /** Returns the config parent of builders or <b>null</b> */
    private static Element getBuildersNode() {
        return getConfigNode(CFG_BUILDER_TAG);
    }


    /** Returns the config parent of parsers or <b>null</b> */
    private static Element getParsersNode() {
        return getConfigNode(CFG_PARSER_TAG);
    }


/**
Make a property builder of a specified name
@param classname fully qualified class name
@return property builder or <b>null</b> if it cannot be created
*/
    public static PropertyBuilder newBuilder(String classname) {
        Object builder = newInstance(classname);
        if (builder != null && !(builder instanceof PropertyBuilder)) builder = null;
        return (PropertyBuilder)builder;
    }

/**
Make an object of a class of a specified name
@param classname fully qualified class name
@return object or <b>null</b> if it cannot be created
*/
    private static Object newInstance(String classname) {
        if (classname == null) return null;
        Object x = null;
        try {
            x = Class.forName(classname).newInstance();
        } catch (ClassNotFoundException c) {
            System.out.println("Cannot find class " + c.getMessage());
        } catch (InstantiationException ie) {
            System.out.println("Instantiation failed: " + ie.getMessage());
        } catch (IllegalAccessException ie) {
            System.out.println("Illegal access: " + ie.getMessage());
        }
        return x;
    }


/**
Make a spreadsheet parser of a specified name
@param classname fully qualified class name
@return spreadsheet parser or <b>null</b> if it cannot be created
*/
    public static SpreadsheetParser newParser(String classname) {
        Object parser = newInstance(classname);
        if (parser != null && !(parser instanceof SpreadsheetParser)) parser = null;
        return (SpreadsheetParser)parser;
    }

    /** Prints the usage. */
    private static void printUsage() {

        System.err.println("usage: java cz.cas.icpf.hroch486.edip.Edip filename");
        System.err.println();

    } // printUsage()

    /** Read the configuration or create a new empty configuration document */
    private static void readConfig() {
        // return if the list already exists
        if (cfgDocument != null) return;
        cfgName = System.getProperty(PARSERS_CONFIG);
        if (cfgName == null) {
            String jhome = System.getProperty("java.home");
            if (jhome == null) {
                System.err.println("Cannot find parser list, system property "
                      + PARSERS_CONFIG + " is not set");
                System.exit(1);
            }
            String sep = System.getProperty("file.separator");
            cfgName = jhome + sep + "lib" + sep + "edipcfg.xml";
        }
        File cfgFile = new File(cfgName);
        if (!cfgFile.exists()) {
            System.err.println("Warning: file " + cfgName + " does not exist");
            cfgDocument = DocumentHelper.createDocument();
            cfgElement = cfgDocument.addElement(CFG_TAG);
        } else {
            try {
                SAXReader xmlReader = new SAXReader();
                cfgDocument = xmlReader.read(cfgFile);
            } catch (DocumentException de) {
                System.err.println("Error reading the configuration file");
                de.printStackTrace();
            }
            // Root element
            cfgElement = cfgDocument.getRootElement();
            String localName = cfgElement.getName();
            String namespace = cfgElement.getNamespaceURI();
            if (!namespace.equals("") || !localName.equals(CFG_TAG)) {
                System.err.println(cfgName + " is not valid parser configuration file.");
                System.err.println(localName + " :: " + namespace);
                System.exit(1);
            }
        }
    }    // end readConfig


/**
Remove spreadsheet parser(s) and property builder(s) from the configuration and write the
configuration to the file
@param names the array of fully qualified class names
*/
    public static void remove(String names[]) {
        readConfig();
        for (int i = 1;  i < names.length;  i++) {
            Object object = newInstance(names[i]);
            if (object instanceof SpreadsheetParser) removeParser(names[i]);
            else if (object instanceof PropertyBuilder) removeBuilderByClassname(names[i]);
        }
        viewConfig();
        writeConfig();
    }


/**
Removes the builder for the specified class
@param classname fully qualified class name
*/
    public static void removeBuilderByClassname(String classname) {
        List builders = findXconfig("/SELFMLconfig/builders/*[@classname='" + classname + "']");
        removeNodes(builders);
    }

/**
Removes the builder for the specified property
@param property property type
*/
    public static void removeBuilderByProperty(String property) {
        List builders = findXconfig("/SELFMLconfig/builders/" + property);
        removeNodes(builders);
    }


/**
Removes the listed nodes from their parents
@param nodeList list of nodes
*/

    private static void removeNodes(List nodeList) {
    for (int i = 0, size = nodeList.size();  i < size;  i++) {
        ((Node)nodeList.get(i)).detach();
    }}

/**
Remove all parsers for the specified class
@param classname fully qualified class name
*/
    public static void removeParser(String classname) {
        List parsers = findXconfig("/SELFMLconfig/parsers/*[@classname='" + classname + "']");
        removeNodes(parsers);
    }


/**
Removes the parser with specified NS URI and element name
@param na namespace URI of the spreadsheet
@param elem name of the spreadsheet root element
*/
    public static void removeParser(String ns, String elem) {
        List parsers = findXconfig("/SELFMLconfig/parsers/" + elem +
                                   "[@ns='" + ns + "']");
        removeNodes(parsers);
    }


    /** View the configuration */
    public static void viewConfig() {
        readConfig();
        viewConfig(cfgDocument);
    }


/**
Internal function which starts viewing configuration
@param doc configuration document
*/

    private static void viewConfig(Document doc) {
        viewConfig(doc.getRootElement());
    }


/**
Internal function for viewing configuration
@param elem configuration element to list
*/

    private static void viewConfig(Element elem) {
        for (int i = 0, size = elem.nodeCount();  i < size;  i++) {
            Node node = elem.node(i);
            if (node instanceof Element) {
                Element ent = (Element)node;
                String elemname = ent.getName();
                if (elemname.equals("parsers") || elemname.equals("builders")) {
                    System.out.print("List of ");
                    if (elemname.equals("parsers")) {
                        System.out.println("spreadsheet parsers");
                        viewConfig(ent, true);
                    } else {
                        System.out.println("property builders");
                        viewConfig(ent, false);
                    }
                    System.out.println();
                }
            }
        }
    }


/**
Internal function for viewing configuration
@param elem configuration element to list
@param isParser true for parsers, false for builders
*/

    private static void viewConfig(Element elem, boolean isParser) {
        for (int i = 0, size = elem.nodeCount();  i < size;  i++) {
            Node node = elem.node(i);
            if (node instanceof Element) {
                Element ent = (Element)node;
                if (isParser) System.out.print(ent.attributeValue("ns") + " :: ");
                System.out.println(ent.getName() + " => " + ent.attributeValue("classname"));
            }
        }
    }


    /** Write the configuration file */
    public static void writeConfig() {
        readConfig();
        if (cfgDocument == null) {
            System.err.println("Fatal error: configuration cannot be created, nothing to write.");
            System.exit(1);
        }
        try {
            FileOutputStream xml = new FileOutputStream(cfgName);
            OutputFormat outformat = OutputFormat.createPrettyPrint();
            outformat.setEncoding("utf-8");
            XMLWriter writer = new XMLWriter(xml, outformat);
            writer.write(cfgDocument);
            writer.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


/** Main entry point */
    public static void main(String argv[]) {
        // is there anything to do?
        if (argv.length == 0) {
            printUsage();
            System.exit(1);
        }
        readConfig();
        if (cfgDocument == null) {
            System.err.println("Fatal error: parser configuration file not found.");
            System.exit(1);
        }
        if (argv[0].equals("-view")) viewConfig();
        else if (argv[0].equals("-add")) add(argv);
        else if (argv[0].equals("-remove")) remove(argv);
        else if (argv[0].equals("-cleanup")) cleanup();
        else {
            boolean outputInternal = false;
            if (argv[0].equals("-internal")) outputInternal = true;
            int arg0 = outputInternal ? 1 : 0;
            Document document = null;
            File xmlFile = new File(argv[arg0]);
            try {
                SAXReader xmlReader = new SAXReader();
                document = xmlReader.read(xmlFile);
            } catch (DocumentException de) {
                System.err.println("Error reading the configuration file");
                de.printStackTrace();
            }

            try {
                // Root element
                Element element = document.getRootElement();
                String localName = element.getName();
                String namespace = element.getNamespaceURI();

                // Internal document does not need parsing
                if (!localName.equals("selfml") || !namespace.equals(INTERNAL_NS)) {
                    SpreadsheetParser parser = newParser(findParser(namespace, localName));
                    if (parser == null)
                        throw new EdipException("No parser for " + namespace
                              + " :: " + localName);

                    document = parser.parse(document);
                    parser = null;   // the parser and all its memory is no longer needed
                }

                // call property builder unless output of the internal file is requested
                if (!outputInternal) {
                    Element selfmlElem = document.getRootElement();
                    PropertyBuilder pb =
                        newBuilder(findBuilder(selfmlElem.attributeValue("property")));
                    if (pb == null)
                        throw new EdipException("No builder for property "
                              + selfmlElem.attributeValue("property"));
                    document = pb.build(document);
                    pb = null;    // the property builder and all its memory is no longer needed
                }

                // write SELFML
                try {
                    int arg1 = arg0 + 1;
                    FileOutputStream xml =
                        new FileOutputStream(argv.length > arg1 ? argv[arg1] : argv[arg0]);
                    OutputFormat outformat = OutputFormat.createPrettyPrint();
                    outformat.setEncoding("utf-8");
                    XMLWriter writer = new XMLWriter(xml, outformat);
                    writer.write(document);
                    writer.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } catch (EdipException e) {
                System.err.println(e.getMessage());
                e.printStackTrace();
            }
        }
    }


} // class Edip
