package cz.cas.icpf.hroch486.selfml.spreadsheet;

import cz.cas.icpf.hroch486.selfml.*;

import java.io.IOException;
import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.DocumentHelper;
import org.dom4j.DocumentException;
import org.dom4j.XPath;

/**
Spreadsheet parser fo MS Office XP Excel
@author Z. Wagner, 2003
*/

public class MSExcelParser implements SpreadsheetParser {

    /** MS Office Spreadsheet XP root element name */
    protected static final String MSSpreadsheetElem = "Workbook";
    /** MS Office Spreadsheet XP namespace */
    protected static final String MSSpreadsheetNS = "urn:schemas-microsoft-com:office:spreadsheet";
    /** MS Excel namespace */
    protected static final String MSExcelNS = "urn:schemas-microsoft-com:office:excel";
    /** MS Office namespace */
    protected static final String MSOfficeNS = "urn:schemas-microsoft-com:office";
    /** namespace map */
    protected Map nsMap;
    /** parsed document */
    protected Document document;

/**
Allocates the map
*/
    public MSExcelParser() {
      nsMap = new HashMap();
      nsMap.put("s", MSSpreadsheetNS);
      nsMap.put("e", MSExcelNS);
      nsMap.put("o", MSOfficeNS);
    }

/**
Finds the named cells of the specified name
@param cellName cell name
@return list of <code>&lt;Data&gt;</code> elements corresponding to the named cells
*/

    private List findNamedCells(String cellName) {
        return searchXPath("//s:Data[../s:NamedCell/@s:Name='" + cellName + "']");
    }


/** Return the name of the spreadsheet root element. */
    public String getElementName() {
        return MSSpreadsheetElem;
    }

/** Return the name of the main spreadsheet namespace URI */
    public String getNamespaceURI() {
        return MSSpreadsheetNS;
    }

/**
Parse the spreadsheet document
@param document the spreadsheet document
@return intermediate document to be used by the property builder
@exception EdipException reason of error
*/
    public Document parse(Document document) throws EdipException {
        System.out.println("Reading the MS Excel spreadsheet");
        this.document = document;

        // Looking for the property type
        List namedCells = findNamedCells("PropertyType");
        if (namedCells.size() != 1) throw new EdipException
           ("There must be exactly one PropertyType cell with a simple string");
        Element elem = (Element)namedCells.get(0);
        document = DocumentHelper.createDocument();
        String ns = Edip.INTERNAL_NS;  // make short name
        Element root = document.addElement("selfml", ns).addAttribute("property", elem.getText());

        // Getting the description block
        verifyNamedCells("DescriptionBlock");
        namedCells = findNamedCells("DescriptionBlock");
        if (namedCells.size() == 0) throw new EdipException("DescriptionBlock is missing");
        Element description = root.addElement("DescriptionBlock", ns);
        Element variables = root.addElement("Variables", ns);
        for (int i = 0, sz = namedCells.size();  i < sz;  i++) {
            elem = (Element)namedCells.get(i);
            String varname = elem.getText();
            if (varname.equals("")) throw new EdipException
               ("DescriptionBlock contains an empty cell.");
            description.addElement("variable", ns).addAttribute("name", varname);
            Element var = variables.addElement("variable", ns).addAttribute("name", varname);
            verifyNamedCells(varname);
            List varlist = findNamedCells(varname);
            if (varlist.size() == 0) throw new EdipException(varname + " cells not found.");
            for (int j = 0, n = varlist.size();  j < n;  j++) {
                elem = (Element)varlist.get(j);
                var.addElement("value", ns).addText(elem.getText());
            }
        }
        return document;
    }


/**
Parses the RxCx specification and returns the row and column numbers
@param spec RxCx specification
@return array containing the row and column number
@exception EdipException error indication
*/

    private int[] parseRowCol(String spec) throws EdipException {
        int rc[] = {0, 0};
        // First character is always ignored
        int Cpos = spec.indexOf('C', 1);
        if (Cpos == -1) throw new EdipException("The spreadsheet is corrupted");
        try {
            rc[0] = Integer.parseInt(spec.substring(1, Cpos));
            rc[1] = Integer.parseInt(spec.substring(Cpos+1));
        } catch (Exception e) {
            throw new EdipException("Corrupted spreadsheet: " + e.getMessage());
        }
        return rc;
    }


/**
Searches the XPath using the namespace map
@param expression XPath expression
@return list of found nodes
*/

    private List searchXPath(String expression) {
        XPath xpathSelector = DocumentHelper.createXPath(expression);
        xpathSelector.setNamespaceURIs(nsMap);
        return xpathSelector.selectNodes(document);
    }


/**
Verifies that the named area is a continuous row or column
@param cellName cell name
@exception EdipException error indication
*/

    private void verifyNamedCells(String cellName) throws EdipException {
        List ranges = searchXPath("//s:NamedRange[@s:Name='" + cellName + "']");
        if (ranges.size() != 1) throw new EdipException ("Named area " + cellName + " not found");
        Element elem = (Element)ranges.get(0);
        String refersTo = elem.attributeValue("RefersTo");

        // Continuous areas have no comma
        if (refersTo.indexOf(',') != -1)
            throw new EdipException ("Named area " + cellName + " is not continuous");

        // Sheet name is followed by !
        int exclamPos = refersTo.indexOf('!');
        if (exclamPos == -1) throw new EdipException ("The spreadsheet is corrupted");

        // Start and end of the area are separated by colon
        int colonPos = refersTo.indexOf(':', exclamPos);
        if (colonPos != -1) {
            int rows = 0, cols = 0;
            try {
                int[] sPos = parseRowCol(refersTo.substring(exclamPos+1, colonPos));
                int[] ePos = parseRowCol(refersTo.substring(colonPos+1));
                rows = ePos[0] - sPos[0];
                cols = ePos[1] - sPos[1];
            } catch (IndexOutOfBoundsException iobe) {
                iobe.printStackTrace(); // this should never happen
            }
            if (rows != 0 && cols != 0) throw new EdipException (cellName +
                                        " must be either a row or a column");
        }
    }


/**
Enables to add or remove the parser
@param argv general command line arguments, only the first argument is used and may be <b>-add</b>
to add the parser or <b>-remove</b> to remove the parser. If not used, <b>-add</b> is assumed.
*/
    public static void main(String argv[]) {
        if (argv.length == 0 || argv[0].equals("-add")) {
            Edip.addParser("cz.cas.icpf.hroch486.selfml.spreadsheet.MSExcelParser",
                            MSSpreadsheetNS, MSSpreadsheetElem);
        } else if (argv.length > 0 && argv[0].equals("-remove")) {
            Edip.removeParser(MSSpreadsheetNS, MSSpreadsheetElem);
        } else System.err.println("Invalid parameters");
    }
}
